<?php

namespace Ems\Services;

use Ems\Repository\{ EstruturaRepository, VinculoRepository };
use Ems\Services\OperadorService;
use Ems\Util\ArrayUtil;
use \Util\Environment;
use Util\Messages;

class EstruturaService
{
    protected $estruturaRepository;
    protected $environement;

    const INDEX_ORDER_ESTRUTURAS_ARRAY = 'NMESTRUTURA';

    public function __construct(
        OperadorService     $operadorService,
        EstruturaRepository $estruturaRepository,
        VinculoRepository   $vinculoRepository,
        Environment         $environment
    ) {
        $this->operadorService      = $operadorService;
        $this->estruturaRepository  = $estruturaRepository;
        $this->vinculoRepository    = $vinculoRepository;
        $this->environment          = $environment;
    }

    /**
     *  Rotina que retorna as estruturas gerenciadas pelo operador
     *
     *  @param string $cdoperador
     *
     *  @return Array $nrEstruturas
     */
    public function getEstruturasGerenciais(
        string $cdOperador
    ) : array {

        $nrEstruturasResp = $this->getEstruturasGerenciaisResp($cdOperador);

        $relacEstruturas = [];

        foreach ($nrEstruturasResp as $nrEstruturaResp) {
            $nrEstruturasInferiores = [];
            $this->getEstruturasInferioresRec($nrEstruturaResp, $nrEstruturasInferiores, false);

            $relacao = [
                'superior'   => $nrEstruturaResp,
                'inferiores' => $nrEstruturasInferiores
            ];

            array_push($relacEstruturas, $relacao);
        }

        $estruturasFiltradas = $this->
            removerEstruturasSeForemInferiores($relacEstruturas);

        $estruturasOrdenadas = $this->
            ordenarEstruturasPorNumeroInferiores($estruturasFiltradas);

        $estruturas = $this->
            planificarRelacEstruturas($estruturasOrdenadas);

        return $estruturas;
    }

    public function getEstruturasInferiores(
        int $nrEstrutura,
        $selfInclude = false
    ) {
        $nrEstruturasInferiores = [];

        $this->getEstruturasInferioresRec(
            $nrEstrutura, $nrEstruturasInferiores, $selfInclude
        );

        return $nrEstruturasInferiores;
    }

    public function getEstruturasInferioresRec(
        int $nrEstrutura,
        array &$children,
        $selfInclude = false
    ) {

        $nrOrg = (int) $this->environment->getNrOrg();

        if (in_array($nrEstrutura, $children)) {
            return;
        }

        if ($selfInclude) {
            array_push($children, $nrEstrutura);
        }

        $paramsFilhas = [
            'P_NRORG'      => $nrOrg,
            'P_NRESTRUTURA' => $nrEstrutura
        ];

        // Todas as estruturas que são *DIRETAMENTE* inferiores à estrutua passada
        $directChildren = $this->getEstruturasGerenciaisFilhas($nrEstrutura);

        foreach ($directChildren as $child) {
            $this->getEstruturasInferioresRec($child, $children, true);
        }
    }


    /**
     *  Rotina que retorna as estruturas imediatamente abaixo ao operador. Caso o operador seja do perfil de acesso administrador do sistema, serão retornadas todas as estruturas gerenciaise legais ativas
     *
     *  @param string $cdoperador
     *
     *  @return Array $nrEstruturas
     */
    public function getEstruturasGerenciaisResp(
        string $cdOperador
    ) : array {

        $nrOrg = (int) $this->environment->getNrOrg();

        // Se for DP, retorna todas as estruturas gerenciais da organização. Caso contrário, são retornadas as estruturas imediatamente abaixo ao operador
        $isOperadorDP = $this->operadorService->operadorIsDP($cdOperador);

        $estruturas = array();
        if ($isOperadorDP) {
            $estruturas = $this->estruturaRepository->getEstruturasAcessiveisDP($nrOrg);
        } else {

            $params = [
                'P_NRORG'      => $nrOrg,
                'P_CDOPERADOR' => $cdOperador
            ];
    
            $estruturas = $this->estruturaRepository->getEstruturasGerenciaisResp($params);
        }


        $nrEstruturas = array_map(function(array $estrutura) {
            $nrEstrutura = (int) $estrutura['NRESTRUTURAM'];

            return $nrEstrutura;
        }, $estruturas);

        return $nrEstruturas;
    }

    public function getEstruturasGerenciaisFilhas(
        int $nrEstrutura
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRESTRUTURA' => $nrEstrutura
        ];

        $estruturas = $this->estruturaRepository->getEstruturasGerenciaisFilhas($params);

        $nrEstruturas = array_map(function(array $estrutura) {
            $nrEstrutura = (int) $estrutura['NRESTRUTURAMREL'];

            return $nrEstrutura;
        }, $estruturas);

        return $nrEstruturas;
    }

    public function getNomeETipoEstrutura(
        int $nrEstrutura
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRESTRUTURA' => $nrEstrutura
        ];

        $result = $this->estruturaRepository->getNomeETipoEstrutura($params);
        $nomeEstrutura = $result[0]['NOME_TIPO_ESTRUTURA'];

        return $nomeEstrutura;
    }

    public function removerEstruturasSeForemInferiores($relacEstruturas) {
        foreach ($relacEstruturas as $key => $relacao) {
            $superior = $relacao['superior'];

            foreach($relacEstruturas as $relac) {
                if ($relac['superior'] !== $superior) {
                    $isInferior = in_array($superior, $relac['inferiores']);

                    if ($isInferior) {
                        unset($relacEstruturas[$key]);
                    }
                }
            }
        }

        return $relacEstruturas;
    }

    public function planificarRelacEstruturas($relacEstruturas) {
        $estruturas = [];

        foreach ($relacEstruturas as $relacao) {
            array_push($estruturas, $relacao['superior']);
            $estruturas = array_merge($estruturas, $relacao['inferiores']);
        }

        return $estruturas;
    }

    public function ordenarEstruturasPorNumeroInferiores($relacEstruturas) {
        usort($relacEstruturas, function($a, $b) {
            $qntInferioresA = count($a['inferiores']);
            $qntInferioresB = count($b['inferiores']);

            $diferenca = $qntInferioresB - $qntInferioresA;

            return $diferenca;
        });

        return $relacEstruturas;
    }
    
    public function getDadosEstrutura(
        string $nrestruturam,
        string $nrorg = null
    ) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $estrutura = $this->estruturaRepository->getEstruturaMestreHistorico($nrestruturam, $nrorg);
        
        if (empty($estrutura)) {
            throw new \Exception(Messages::ESTRUT_NOT_FOUND);
        }
        
        return $estrutura;
    }
    
    public function retornaDadosResponsavelInformal($nrorg, $nrestrutgeren, $dtmescompetenc){    
        return $this->estruturaRepository->retornaDadosResponsavelInformal($nrorg, $nrestrutgeren, $dtmescompetenc);
    }  
}
