<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;
use Util\Messages;

use Ems\Util\Utilities;
use Ems\Services\EmailService;
use Ems\Repository\FeriasRepository;
use Ems\Repository\PeriodoRepository;

class FeriasService {
    
    public function __construct(EntityManager $entityManager, Environment $environment, EmailService $emailService, FeriasRepository $feriasRepository, PeriodoRepository $periodoRepository) {
        $this->entityManager        = $entityManager; 
        $this->environment          = $environment;
        $this->emailService         = $emailService;
        $this->feriasRepository     = $feriasRepository;
        $this->periodoRepository    = $periodoRepository;
    }
    
    protected const PROGFERIAS_OMITIRAGRUP     = 'N';
    protected const PROGFERIAS_VINCAFAST       = 'S';
    protected const PROGFERIAS_NRTIPOHIERARQ   = '1';
    
    protected const DISPARA_EMAIL_RH_ALTERACAOPROGFERIAS = true;
    
    // Separar isso em um service proprio
    protected const VALIDACOES_PROGFERIAS = array(
        array(
            'name'          => 'VALIDA_DATA_INICIAL_FINAL_PROGRAMACAO',
            'method'        => 'validacaoDataInicialFinalProgramacaoFerias'
        ),
        array(
            'name'          => 'VALIDA_QNT_DIAS_MAXMIN_FERIAS',
            'method'        => 'validacaoDiasMinMaxFerias'
        )
    );
    
    public function getPrevisaoFerias(
        string $dtmescompetenc,
        string $nrtipohierarq,
        string $vinculo         = null,
        string $estrutgeren     = null,
        string $estrutlegal     = null,
        string $meslimite       = null,
        string $omitiragrup     = null,
        string $vincafast       = null
    ): Array {
        
        $nrorg = $this->environment->getNrorg();
        $nrorgpadrao = $this->environment->getNrorgPadrao();
        
        return $this->feriasRepository->getPrevisaoFerias(
            $nrorg,
            $nrorgpadrao,
            $dtmescompetenc,
            $nrtipohierarq,
            $vinculo,
            $estrutgeren,
            $estrutlegal,
            $meslimite,
            $omitiragrup,
            $vincafast
        );
    }
    
    public function getProgramacaoFerias(
        string $nrperiodoapuracao   = null,
        string $vinculo             = null,
        string $estrutgeren         = null,
        string $estrutlegal         = null,
        string $meslimite           = null,
        string $nrtipohierarq       = self::PROGFERIAS_NRTIPOHIERARQ,
        string $omitiragrup         = self::PROGFERIAS_OMITIRAGRUP,
        string $vincafast           = self::PROGFERIAS_VINCAFAST
    ): Array {
        $nrorg = $this->environment->getNrorg();
        $dtmescompetenc = $this->periodoRepository->getDtmescompetencByPeriodo($nrorg, $nrperiodoapuracao);
        
        return $this->getPrevisaoFerias(
            $dtmescompetenc,
            $nrtipohierarq,
            $vinculo,
            $estrutgeren,
            $estrutlegal,
            $meslimite,
            $omitiragrup,
            $vincafast
        );
    }
    
    /**
     *  Rotina que permite o registro de PREVISÕES DE FÉRIAS. Pode ser parametrizada para enviar emails alertando os adminstradores de DP/RH
     * 
     *  @param string $nrferias,
     *  @param string $nrvinculom,
     *  @param string $dtiniprevisao       Default: null
     *  @param string $dtfinprevisao       Default: null
     *  @param string $qtdiasprevisao      Default: null
     *  @param string $dtiniprevisao2      Default: null
     *  @param string $dtfinprevisao2      Default: null
     *  @param string $qtdiasprevisao2     Default: null
     *  @param string $dtiniprevisao3      Default: null
     *  @param string $dtfinprevisao3      Default: null
     *  @param string $qtdiasprevisao3     Default: null
     *  @param string $cdoperador          Default: null
     *  @param bool   $disparaEmailDpRh    Default: false
     * 
     *  @return bool $success
     */
    public function setProgramacaoFerias(
        string $nrferias,
        string $nrvinculom,
        string $dtiniprevisao       = null,
        string $dtfinprevisao       = null,
        string $qtdiasprevisao      = null,
        string $qtabonoprevisao     = null,
        string $dtiniprevisao2      = null,
        string $dtfinprevisao2      = null,
        string $qtdiasprevisao2     = null,
        string $qtabonoprevisao2    = null,
        string $dtiniprevisao3      = null,
        string $dtfinprevisao3      = null,
        string $qtdiasprevisao3     = null,
        string $qtabonoprevisao3    = null,
        string $cdoperador          = null,
        bool   $disparaEmailDpRh    = false
    ) {
        $nrorg = $this->environment->getNrorg();
        
        $programacaoFerias = array(
            'NRFERIAS'          => $nrferias,
            'NRVINCULOM'        => $nrvinculom,
            'DTINIPREVISAO'     => $dtiniprevisao,
            'DTFINPREVISAO'     => $dtfinprevisao,
            'QTDIASPREVISAO'    => $qtdiasprevisao,
            'QTABONOPREVISAO'   => $qtabonoprevisao,
            'DTINIPREVISAO2'    => $dtiniprevisao2,
            'DTFINPREVISAO2'    => $dtfinprevisao2,
            'QTDIASPREVISAO2'   => $qtdiasprevisao2,
            'QTABONOPREVISAO2'  => $qtabonoprevisao2,
            'DTINIPREVISAO3'    => $dtiniprevisao3,
            'DTFINPREVISAO3'    => $dtfinprevisao3,
            'QTDIASPREVISAO3'   => $qtdiasprevisao3,
            'QTABONOPREVISAO3'  => $qtabonoprevisao3,
        );
        
        // Validações sobre a programação de férias
        $programacaoFerias = $this->validaProgramacaoFerias(
            $programacaoFerias
        );
        
        $success = $this->feriasRepository->setProgramacaoFerias(
            $nrorg,
            $programacaoFerias['NRFERIAS'],
            $programacaoFerias['NRVINCULOM'],
            $programacaoFerias['DTINIPREVISAO'],
            $programacaoFerias['DTFINPREVISAO'],
            $programacaoFerias['QTDIASPREVISAO'],
            $programacaoFerias['QTABONOPREVISAO'],
            $programacaoFerias['DTINIPREVISAO2'],
            $programacaoFerias['DTFINPREVISAO2'],
            $programacaoFerias['QTDIASPREVISAO2'],
            $programacaoFerias['QTABONOPREVISAO2'],
            $programacaoFerias['DTINIPREVISAO3'],
            $programacaoFerias['DTFINPREVISAO3'],
            $programacaoFerias['QTDIASPREVISAO3'],
            $programacaoFerias['QTABONOPREVISAO3']
        );
        
        if ($success && $disparaEmailDpRh && !empty($cdoperador)) {
            // Dispara email com as alterações realizadas
            $emailsEnviados = $this->enviaEmailAlteracaoProgramacaoFerias($programacaoFerias, $cdoperador);

            // Se os emails nao foram enviados, $emailsEnviados contem mais detalhes
            if ($emailsEnviados !== true) {
                $exceptionMessage = $this->emailService->agrupaMensagensEmail($emailsEnviados);
                throw new \Exception($exceptionMessage);
            }
        }
        
        return $success;
    }
    
    /**
     *  Rotina que permite o registro de PREVISÕES DE FÉRIAS. Pode ser parametrizada para enviar emails alertando os adminstradores de DP/RH
     * 
     *  @param Array $rows
     *      Format:
     *          [
     *              [
     *                  'NRFERIAS'          => $value,
     *                  'NRVINCULOM'        => $value,
     *                  'DTINIPREVISAO'     => $value,
     *                  'DTFINPREVISAO'     => $value,
     *                  'QTDIASPREVISAO'    => $value,
     *                  'DTINIPREVISAO2'    => $value,
     *                  'DTFINPREVISAO2'    => $value,
     *                  'QTDIASPREVISAO2'   => $value,
     *                  'DTINIPREVISAO3'    => $value,
     *                  'DTFINPREVISAO3'    => $value,
     *                  'QTDIASPREVISAO3'   => $value
     *              ]
     *          ]
     * 
     *  @return bool $success
     */
    public function setMultiplasProgramacaoFerias(
        Array  $rows,
        string $cdoperador          = null,
        bool   $disparaEmailDpRh    = self::DISPARA_EMAIL_RH_ALTERACAOPROGFERIAS
    ) {
        if (!is_array($rows)) {
            return false;
        }
        
        
        foreach ($rows as $row) {
            $nrferias           = $row['NRFERIAS'];
            $nrvinculom         = $row['NRVINCULOM'];
            $dtiniprevisao      = isset($row['DTINIPREVISAO'])     ? $row['DTINIPREVISAO']     : null;
            $dtfinprevisao      = isset($row['DTFINPREVISAO'])     ? $row['DTFINPREVISAO']     : null;
            $qtdiasprevisao     = isset($row['QTDIASPREVISAO'])    ? $row['QTDIASPREVISAO']    : null;
            $qtabonoprevisao    = isset($row['QTABONOPREVISAO'])   ? $row['QTABONOPREVISAO']   : null;
            $dtiniprevisao2     = isset($row['DTINIPREVISAO2'])    ? $row['DTINIPREVISAO2']    : null;
            $dtfinprevisao2     = isset($row['DTFINPREVISAO2'])    ? $row['DTFINPREVISAO2']    : null;
            $qtdiasprevisao2    = isset($row['QTDIASPREVISAO2'])   ? $row['QTDIASPREVISAO2']   : null;
            $qtabonoprevisao2   = isset($row['QTABONOPREVISAO2'])  ? $row['QTABONOPREVISAO2']  : null;
            $dtiniprevisao3     = isset($row['DTINIPREVISAO3'])    ? $row['DTINIPREVISAO3']    : null;
            $dtfinprevisao3     = isset($row['DTFINPREVISAO3'])    ? $row['DTFINPREVISAO3']    : null;
            $qtdiasprevisao3    = isset($row['QTDIASPREVISAO3'])   ? $row['QTDIASPREVISAO3']   : null;
            $qtabonoprevisao3   = isset($row['QTABONOPREVISAO3'])  ? $row['QTABONOPREVISAO3']  : null;
            
            $success = $this->setProgramacaoFerias(
                $nrferias,
                $nrvinculom,
                $dtiniprevisao,
                $dtfinprevisao,
                $qtdiasprevisao,
                $qtabonoprevisao,
                $dtiniprevisao2,
                $dtfinprevisao2,
                $qtdiasprevisao2,
                $qtabonoprevisao2,
                $dtiniprevisao3,
                $dtfinprevisao3,
                $qtdiasprevisao3,
                $qtabonoprevisao3,
                $cdoperador,
                $disparaEmailDpRh
            );
            
            if (empty($success)) {
                throw new \Exception(Messages::FERIAS_PROGFERIAS_NAO_SALVA);
            }
        }
        
        return true;
    }
    
    /**
     *  Rotina que dispara a rotina de email ao salvar uma PROGRAMAÇÃO DE FÉRIAS, previamente parametrizada para acionar essa rotina.
     * 
     *  @param Array $programacaoFerias
     *      Format:
     *          'NRFERIAS'          => $value,
     *          'NRVINCULOM'        => $value,
     *          'DTINIPREVISAO'     => $value,
     *          'DTFINPREVISAO'     => $value,
     *          'QTDIASPREVISAO'    => $value,
     *          'DTINIPREVISAO2'    => $value,
     *          'DTFINPREVISAO2'    => $value,
     *          'QTDIASPREVISAO2'   => $value,
     *          'DTINIPREVISAO3'    => $value,
     *          'DTFINPREVISAO3'    => $value,
     *          'QTDIASPREVISAO3'   => $value
     */
    public function enviaEmailAlteracaoProgramacaoFerias(
        Array  $programacaoFerias,
        string $cdoperador
    ) {
        
        return $this->emailService->sendEmailNotificacaoProgramacaoFerias(
            $cdoperador,
            array(
                'NRVINCULOM'        => $programacaoFerias['NRVINCULOM'],
                'DTINIPREVISAO'     => $programacaoFerias['DTINIPREVISAO'],
                'QTDIASPREVISAO'    => $programacaoFerias['QTDIASPREVISAO'],
                'DTFINPREVISAO'     => $programacaoFerias['DTFINPREVISAO'],
                'QTABONOPREVISAO'   => $programacaoFerias['QTABONOPREVISAO']
            )
        );
    }
    
    /**
     *  Rotina que valida as informações de data serem inseridas como PROGRAMAÇÃO DE FÉRIAS.
     * 
     *  @param Array $programacaoFerias
     *      Format:
     *          'NRFERIAS'          => $value,
     *          'DTINIPREVISAO'     => $value,
     *          'DTFINPREVISAO'     => $value,
     *          'QTDIASPREVISAO'    => $value,
     *          'DTINIPREVISAO2'    => $value,
     *          'DTFINPREVISAO2'    => $value,
     *          'QTDIASPREVISAO2'   => $value,
     *          'DTINIPREVISAO3'    => $value,
     *          'DTFINPREVISAO3'    => $value,
     *          'QTDIASPREVISAO3'   => $value
     *  
     *  @return Array $programacaoFeriasValidado
     *      Format: Mesmo de entrada
     */
    public function validacaoDataInicialFinalProgramacaoFerias(
        Array $programacaoFerias
    ) {
        $dataFimPrevisao = null;
        if(!empty($programacaoFerias['DTINIPREVISAO']) && !is_null($programacaoFerias['QTDIASPREVISAO']) && $programacaoFerias['QTDIASPREVISAO'] !== '') {
            $qntDiasPrevisao = is_null($programacaoFerias['QTDIASPREVISAO']) ? 0 : $programacaoFerias['QTDIASPREVISAO'];
            $dataFimPrevisao = $this->projetaDataFimProgramacaoFerias($programacaoFerias['DTINIPREVISAO'], $qntDiasPrevisao);
        }
        
        $dataFimPrevisao2 = null;
        if(!empty($programacaoFerias['DTINIPREVISAO2']) && !is_null($programacaoFerias['QTDIASPREVISAO2']) && $programacaoFerias['QTDIASPREVISAO2'] !== '') {
            $qntDiasPrevisao2 = is_null($programacaoFerias['QTDIASPREVISAO2']) ? 0 : $programacaoFerias['QTDIASPREVISAO2'];
            $dataFimPrevisao2 = $this->projetaDataFimProgramacaoFerias($programacaoFerias['DTINIPREVISAO2'], $qntDiasPrevisao2);
        }
        
        
        $dataFimPrevisao3 = null;
        if(!empty($programacaoFerias['DTINIPREVISAO3']) && !is_null($programacaoFerias['QTDIASPREVISAO3']) && $programacaoFerias['QTDIASPREVISAO3'] !== '') {
            $qntDiasPrevisao3 = is_null($programacaoFerias['QTDIASPREVISAO3']) ? 0 : $programacaoFerias['QTDIASPREVISAO3'];
            $dataFimPrevisao3 = $this->projetaDataFimProgramacaoFerias($programacaoFerias['DTINIPREVISAO3'], $qntDiasPrevisao3);
        }
        
        $validacaoDataInicialFinal = array(
            'DTFINPREVISAO' => $dataFimPrevisao,
            'DTFINPREVISAO2' => $dataFimPrevisao2,
            'DTFINPREVISAO3' => $dataFimPrevisao3,
        );
        
        $programacaoFeriasValidado = array_merge($programacaoFerias, $validacaoDataInicialFinal);
        
        return $programacaoFeriasValidado;
        
    }
    
    public function validacaoDiasMinMaxFerias(
        Array $programacaoFerias
    ) {
        $nrorg = $this->environment->getNrorg();
        $nrorgpadrao = $this->environment->getNrorgPadrao();
        
        $nrferias = $programacaoFerias['NRFERIAS'];
        
        $qtdiasprevisao     = $programacaoFerias['QTDIASPREVISAO'];
        $qtdiasprevisao2    = $programacaoFerias['QTDIASPREVISAO2'];
        $qtdiasprevisao3    = $programacaoFerias['QTDIASPREVISAO3'];
        
        $qtabonoprevisao     = $programacaoFerias['QTABONOPREVISAO'];
        $qtabonoprevisao2    = $programacaoFerias['QTABONOPREVISAO2'];
        $qtabonoprevisao3    = $programacaoFerias['QTABONOPREVISAO3'];
        
        // Não precisa validar
        if (empty($qtdiasprevisao) && empty($qtdiasprevisao2) && empty($qtdiasprevisao3)) {
            return $programacaoFerias;
        }
        
        $diasAValidar = array(
            array(
                'QTDIASPREVISAO'    => $qtdiasprevisao,
                'QTABONOPREVISAO'   => $qtabonoprevisao
            ),
            array(
                'QTDIASPREVISAO'    => $qtdiasprevisao2,
                'QTABONOPREVISAO'   => $qtabonoprevisao2
            ),
            array(
                'QTDIASPREVISAO'    => $qtdiasprevisao3,
                'QTABONOPREVISAO'   => $qtabonoprevisao3
            )
        );
        
        $tpferias = $this->feriasRepository->getTpFeriasByNrferias($nrorg, $nrorgpadrao, $nrferias);
        
        $minimoDias     = (int) $tpferias['QTDIASMINFE'];
        $maximoDias     = (int) $tpferias['QTDIASMAXFE'];
        $diasRestantes  = (int) $tpferias['DIAS_RESTANTE_GOZO'];
        
        $totalDias = $maximoDias - $diasRestantes;
        foreach ($diasAValidar as $dia) {
            $totalDias += (int) $dia['QTDIASPREVISAO'] + (int) $dia['QTABONOPREVISAO'];
        }
        
        if ($totalDias > $maximoDias || $totalDias < $minimoDias) {
            throw new \Exception(Messages::FERIAS_PROGFERIAS_QNT_DIAS_SUPERIOR_LIMITE);
        }
        
        return $programacaoFerias;
    }
    
    /**
     *  Função que retorna a data fim de PROGRAMAÇÃO DE FÉRIAS a partir da data inicial e de uma quantidade de dias.
     * 
     *  @param string $dataInicial
     *  @param string $qntDias
     * 
     *  @return $dataFim
     */
    public function projetaDataFimProgramacaoFerias(
        string $dataInicial,
        string $qntDias
    ) {
        $qntDiasNormalized = empty($qntDias) ? 0 : (int) $qntDias;
        
        $qntDiasASomar = $qntDiasNormalized == 0 ? $qntDiasNormalized : $qntDiasNormalized - 1;
        $dataInicio = DateUtil::getDataDeString($dataInicial);
        $dataFim = DateUtil::adicionaIntervalo($dataInicio, $qntDiasASomar)->format(DateUtil::FORMATO_DATA);
        
        return $dataFim;
    }
    
    /**
     * Rotina que percorre o array de validações acionando as rotinas.
     * 
     *  @param Array $programacaoFerias
     *      Format:
     *          'NRFERIAS'          => $value,
     *          'DTINIPREVISAO'     => $value,
     *          'DTFINPREVISAO'     => $value,
     *          'QTDIASPREVISAO'    => $value,
     *          'DTINIPREVISAO2'    => $value,
     *          'DTFINPREVISAO2'    => $value,
     *          'QTDIASPREVISAO2'   => $value,
     *          'DTINIPREVISAO3'    => $value,
     *          'DTFINPREVISAO3'    => $value,
     *          'QTDIASPREVISAO3'   => $value
     *  
     *  @return Array $programacaoFeriasValidado
     *      Format: Mesmo de entrada
     * 
     *  @throws Exception
     */
    public function validaProgramacaoFerias(
        Array $programacaoFerias    
    ) {
        $validacoes = self::VALIDACOES_PROGFERIAS;
        
        foreach ($validacoes as $validacao) {
            $validationMethod = $validacao['method'];
            
            $programacaoFerias = $this->$validationMethod($programacaoFerias);
        }
        
        return $programacaoFerias;
    }
    
}